@@ -58,7 +58,9 @@ def test_parse_configuration_transforms_file_into_mapping():
         '''
     )
 
-    config, config_paths, logs = module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    config, config_paths, logs = module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
     assert config == {
         'source_directories': ['/home', '/etc'],
@@ -67,6 +69,7 @@ def test_parse_configuration_transforms_file_into_mapping():
         'keep_hourly': 24,
         'keep_minutely': 60,
         'checks': [{'name': 'repository'}, {'name': 'archives'}],
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/config.yaml'}
     assert logs == []
@@ -85,11 +88,14 @@ def test_parse_configuration_passes_through_quoted_punctuation():
         '''
     )
 
-    config, config_paths, logs = module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    config, config_paths, logs = module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
     assert config == {
         'source_directories': [f'/home/{string.punctuation}'],
         'repositories': [{'path': 'test.borg'}],
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/config.yaml'}
     assert logs == []
@@ -117,7 +123,9 @@ def test_parse_configuration_with_schema_lacking_examples_does_not_raise():
         ''',
     )
 
-    module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
 
 def test_parse_configuration_inlines_include_inside_deprecated_section():
@@ -143,13 +151,16 @@ def test_parse_configuration_inlines_include_inside_deprecated_section():
     include_file.name = 'include.yaml'
     builtins.should_receive('open').with_args('/tmp/include.yaml').and_return(include_file)
 
-    config, config_paths, logs = module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    config, config_paths, logs = module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
     assert config == {
         'source_directories': ['/home'],
         'repositories': [{'path': 'hostname.borg'}],
         'keep_daily': 7,
         'keep_hourly': 24,
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/include.yaml', '/tmp/config.yaml'}
     assert len(logs) == 1
@@ -178,13 +189,16 @@ def test_parse_configuration_merges_include():
     include_file.name = 'include.yaml'
     builtins.should_receive('open').with_args('/tmp/include.yaml').and_return(include_file)
 
-    config, config_paths, logs = module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    config, config_paths, logs = module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
     assert config == {
         'source_directories': ['/home'],
         'repositories': [{'path': 'hostname.borg'}],
         'keep_daily': 1,
         'keep_hourly': 24,
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/include.yaml', '/tmp/config.yaml'}
     assert logs == []
@@ -192,7 +206,9 @@ def test_parse_configuration_merges_include():
 
 def test_parse_configuration_raises_for_missing_config_file():
     with pytest.raises(FileNotFoundError):
-        module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+        module.parse_configuration(
+            '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+        )
 
 
 def test_parse_configuration_raises_for_missing_schema_file():
@@ -204,14 +220,18 @@ def test_parse_configuration_raises_for_missing_schema_file():
     builtins.should_receive('open').with_args('/tmp/schema.yaml').and_raise(FileNotFoundError)
 
     with pytest.raises(FileNotFoundError):
-        module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+        module.parse_configuration(
+            '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+        )
 
 
 def test_parse_configuration_raises_for_syntax_error():
     mock_config_and_schema('foo:\nbar')
 
     with pytest.raises(ValueError):
-        module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+        module.parse_configuration(
+            '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+        )
 
 
 def test_parse_configuration_raises_for_validation_error():
@@ -224,7 +244,9 @@ def test_parse_configuration_raises_for_validation_error():
     )
 
     with pytest.raises(module.Validation_error):
-        module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+        module.parse_configuration(
+            '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+        )
 
 
 def test_parse_configuration_applies_overrides():
@@ -241,13 +263,17 @@ def test_parse_configuration_applies_overrides():
     )
 
     config, config_paths, logs = module.parse_configuration(
-        '/tmp/config.yaml', '/tmp/schema.yaml', overrides=['local_path=borg2']
+        '/tmp/config.yaml',
+        '/tmp/schema.yaml',
+        arguments={'global': flexmock()},
+        overrides=['local_path=borg2'],
     )
 
     assert config == {
         'source_directories': ['/home'],
         'repositories': [{'path': 'hostname.borg'}],
         'local_path': 'borg2',
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/config.yaml'}
     assert logs == []
@@ -268,12 +294,15 @@ def test_parse_configuration_applies_normalization_after_environment_variable_in
     )
     flexmock(os).should_receive('getenv').replace_with(lambda variable_name, default: default)
 
-    config, config_paths, logs = module.parse_configuration('/tmp/config.yaml', '/tmp/schema.yaml')
+    config, config_paths, logs = module.parse_configuration(
+        '/tmp/config.yaml', '/tmp/schema.yaml', arguments={'global': flexmock()}
+    )
 
     assert config == {
         'source_directories': ['/home'],
         'repositories': [{'path': 'ssh://user@hostname/./repo'}],
         'exclude_if_present': ['.nobackup'],
+        'bootstrap': {},
     }
     assert config_paths == {'/tmp/config.yaml'}
     assert logs
