@@ -21,8 +21,8 @@ def test_make_delete_command_includes_log_info():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -43,8 +43,8 @@ def test_make_delete_command_includes_log_debug():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -67,8 +67,8 @@ def test_make_delete_command_includes_dry_run():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=True, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=True),
         local_path='borg',
         remote_path=None,
     )
@@ -91,8 +91,8 @@ def test_make_delete_command_includes_remote_path():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path='borg1',
     )
@@ -114,8 +114,8 @@ def test_make_delete_command_includes_umask():
         repository={'path': 'repo'},
         config={'umask': '077'},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -136,10 +136,10 @@ def test_make_delete_command_includes_log_json():
 
     command = module.make_delete_command(
         repository={'path': 'repo'},
-        config={},
+        config={'log_json': True},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=True),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -162,8 +162,8 @@ def test_make_delete_command_includes_lock_wait():
         repository={'path': 'repo'},
         config={'lock_wait': 5},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -171,7 +171,7 @@ def test_make_delete_command_includes_lock_wait():
     assert command == ('borg', 'delete', '--lock-wait', '5', 'repo')
 
 
-def test_make_delete_command_includes_list():
+def test_make_delete_command_with_list_config_calls_borg_with_list_flag():
     flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
     flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
         'list', True
@@ -184,10 +184,10 @@ def test_make_delete_command_includes_list():
 
     command = module.make_delete_command(
         repository={'path': 'repo'},
-        config={},
+        config={'list_details': True},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=True, force=0, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=None, force=0, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -207,8 +207,8 @@ def test_make_delete_command_includes_force():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=1, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=1, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -228,8 +228,8 @@ def test_make_delete_command_includes_force_twice():
         repository={'path': 'repo'},
         config={},
         local_borg_version='1.2.3',
-        delete_arguments=flexmock(list_archives=False, force=2, match_archives=None, archive=None),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        delete_arguments=flexmock(list_details=False, force=2, match_archives=None, archive=None),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -252,9 +252,9 @@ def test_make_delete_command_includes_archive():
         config={},
         local_borg_version='1.2.3',
         delete_arguments=flexmock(
-            list_archives=False, force=0, match_archives=None, archive='archive'
+            list_details=False, force=0, match_archives=None, archive='archive'
         ),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -277,9 +277,9 @@ def test_make_delete_command_includes_match_archives():
         config={},
         local_borg_version='1.2.3',
         delete_arguments=flexmock(
-            list_archives=False, force=0, match_archives='sh:foo*', archive='archive'
+            list_details=False, force=0, match_archives='sh:foo*', archive='archive'
         ),
-        global_arguments=flexmock(dry_run=False, log_json=False),
+        global_arguments=flexmock(dry_run=False),
         local_path='borg',
         remote_path=None,
     )
@@ -287,8 +287,12 @@ def test_make_delete_command_includes_match_archives():
     assert command == ('borg', 'delete', '--match-archives', 'sh:foo*', 'repo')
 
 
+LOGGING_ANSWER = flexmock()
+
+
 def test_delete_archives_with_archive_calls_borg_delete():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = LOGGING_ANSWER
     flexmock(module.borgmatic.borg.repo_delete).should_receive('delete_repository').never()
     flexmock(module).should_receive('make_delete_command').and_return(flexmock())
     flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
@@ -308,6 +312,7 @@ def test_delete_archives_with_archive_calls_borg_delete():
 
 def test_delete_archives_with_match_archives_calls_borg_delete():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = LOGGING_ANSWER
     flexmock(module.borgmatic.borg.repo_delete).should_receive('delete_repository').never()
     flexmock(module).should_receive('make_delete_command').and_return(flexmock())
     flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
@@ -328,6 +333,7 @@ def test_delete_archives_with_match_archives_calls_borg_delete():
 @pytest.mark.parametrize('argument_name', module.ARCHIVE_RELATED_ARGUMENT_NAMES[2:])
 def test_delete_archives_with_archive_related_argument_calls_borg_delete(argument_name):
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = LOGGING_ANSWER
     flexmock(module.borgmatic.borg.repo_delete).should_receive('delete_repository').never()
     flexmock(module).should_receive('make_delete_command').and_return(flexmock())
     flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
@@ -347,6 +353,7 @@ def test_delete_archives_with_archive_related_argument_calls_borg_delete(argumen
 
 def test_delete_archives_without_archive_related_argument_calls_borg_repo_delete():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = LOGGING_ANSWER
     flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
     flexmock(module.borgmatic.borg.repo_delete).should_receive('delete_repository').once()
     flexmock(module).should_receive('make_delete_command').never()
@@ -359,7 +366,7 @@ def test_delete_archives_without_archive_related_argument_calls_borg_repo_delete
         config={},
         local_borg_version=flexmock(),
         delete_arguments=flexmock(
-            list_archives=True, force=False, cache_only=False, keep_security_info=False
+            list_details=True, force=False, cache_only=False, keep_security_info=False
         ),
         global_arguments=flexmock(),
     )
@@ -367,12 +374,13 @@ def test_delete_archives_without_archive_related_argument_calls_borg_repo_delete
 
 def test_delete_archives_calls_borg_delete_with_working_directory():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = LOGGING_ANSWER
     flexmock(module.borgmatic.borg.repo_delete).should_receive('delete_repository').never()
     command = flexmock()
     flexmock(module).should_receive('make_delete_command').and_return(command)
-    extra_environment = flexmock()
+    environment = flexmock()
     flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
-        extra_environment
+        environment
     )
     flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(
         '/working/dir'
@@ -380,7 +388,7 @@ def test_delete_archives_calls_borg_delete_with_working_directory():
     flexmock(module.borgmatic.execute).should_receive('execute_command').with_args(
         command,
         output_log_level=logging.ANSWER,
-        extra_environment=extra_environment,
+        environment=environment,
         working_directory='/working/dir',
         borg_local_path='borg',
         borg_exit_codes=None,
