@@ -20,7 +20,7 @@ def test_make_info_command_constructs_borg_info_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -42,7 +42,7 @@ def test_make_info_command_with_log_info_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -64,7 +64,7 @@ def test_make_info_command_with_log_info_and_json_omits_borg_logging_flags():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=True, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -86,7 +86,7 @@ def test_make_info_command_with_log_debug_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -107,7 +107,7 @@ def test_make_info_command_with_log_debug_and_json_omits_borg_logging_flags():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=True, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -128,7 +128,7 @@ def test_make_info_command_with_json_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=True, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -149,7 +149,7 @@ def test_make_info_command_with_archive_uses_match_archives_flags():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive='archive', json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -170,7 +170,7 @@ def test_make_info_command_with_local_path_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg1',
         remote_path=None,
@@ -194,7 +194,7 @@ def test_make_info_command_with_remote_path_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path='borg1',
@@ -203,6 +203,29 @@ def test_make_info_command_with_remote_path_passes_through_to_command():
     assert command == ('borg', 'info', '--remote-path', 'borg1', '--repo', 'repo')
 
 
+def test_make_info_command_with_umask_passes_through_to_command():
+    flexmock(module.flags).should_receive('make_flags').replace_with(
+        lambda name, value: (f'--{name}', value) if value else ()
+    )
+    flexmock(module.flags).should_receive('make_match_archives_flags').with_args(
+        None, None, '2.3.4'
+    ).and_return(())
+    flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(())
+    flexmock(module.flags).should_receive('make_repository_flags').and_return(('--repo', 'repo'))
+
+    command = module.make_info_command(
+        repository_path='repo',
+        config={'umask': '077'},
+        local_borg_version='2.3.4',
+        global_arguments=flexmock(),
+        info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
+        local_path='borg',
+        remote_path=None,
+    )
+
+    assert command == ('borg', 'info', '--umask', '077', '--repo', 'repo')
+
+
 def test_make_info_command_with_log_json_passes_through_to_command():
     flexmock(module.flags).should_receive('make_flags').and_return(())
     flexmock(module.flags).should_receive('make_flags').with_args('log-json', True).and_return(
@@ -216,9 +239,9 @@ def test_make_info_command_with_log_json_passes_through_to_command():
 
     command = module.make_info_command(
         repository_path='repo',
-        config={},
+        config={'log_json': True},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=True),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -243,7 +266,7 @@ def test_make_info_command_with_lock_wait_passes_through_to_command():
         repository_path='repo',
         config=config,
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -267,7 +290,7 @@ def test_make_info_command_transforms_prefix_into_match_archives_flags():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix='foo'),
         local_path='borg',
         remote_path=None,
@@ -291,7 +314,7 @@ def test_make_info_command_prefers_prefix_over_archive_name_format():
         repository_path='repo',
         config={'archive_name_format': 'bar-{now}'},  # noqa: FS003
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix='foo'),
         local_path='borg',
         remote_path=None,
@@ -312,7 +335,7 @@ def test_make_info_command_transforms_archive_name_format_into_match_archives_fl
         repository_path='repo',
         config={'archive_name_format': 'bar-{now}'},  # noqa: FS003
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -337,7 +360,7 @@ def test_make_info_command_with_match_archives_option_passes_through_to_command(
             'match_archives': 'sh:foo-*',
         },
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
         local_path='borg',
         remote_path=None,
@@ -357,9 +380,9 @@ def test_make_info_command_with_match_archives_flag_passes_through_to_command():
 
     command = module.make_info_command(
         repository_path='repo',
-        config={'archive_name_format': 'bar-{now}'},  # noqa: FS003
+        config={'archive_name_format': 'bar-{now}', 'match_archives': 'sh:foo-*'},  # noqa: FS003
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives='sh:foo-*'),
         local_path='borg',
         remote_path=None,
@@ -385,7 +408,7 @@ def test_make_info_command_passes_arguments_through_to_command(argument_name):
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(
             archive=None, json=False, prefix=None, match_archives=None, **{argument_name: 'value'}
         ),
@@ -420,7 +443,7 @@ def test_make_info_command_with_date_based_matching_passes_through_to_command():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=info_arguments,
         local_path='borg',
         remote_path=None,
@@ -446,6 +469,7 @@ def test_display_archives_info_calls_two_commands():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
     flexmock(module).should_receive('make_info_command')
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command_and_capture_output').once()
     flexmock(module.flags).should_receive('warn_for_aggressive_archive_flags')
     flexmock(module).should_receive('execute_command').once()
@@ -454,7 +478,7 @@ def test_display_archives_info_calls_two_commands():
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
     )
 
@@ -463,6 +487,7 @@ def test_display_archives_info_with_json_calls_json_command_only():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
     flexmock(module).should_receive('make_info_command')
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     json_output = flexmock()
     flexmock(module).should_receive('execute_command_and_capture_output').and_return(json_output)
     flexmock(module.flags).should_receive('warn_for_aggressive_archive_flags').never()
@@ -473,8 +498,41 @@ def test_display_archives_info_with_json_calls_json_command_only():
             repository_path='repo',
             config={},
             local_borg_version='2.3.4',
-            global_arguments=flexmock(log_json=False),
+            global_arguments=flexmock(),
             info_arguments=flexmock(archive=None, json=True, prefix=None, match_archives=None),
         )
         == json_output
     )
+
+
+def test_display_archives_info_calls_borg_with_working_directory():
+    flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module).should_receive('make_info_command')
+    flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(
+        '/working/dir',
+    )
+    flexmock(module).should_receive('execute_command_and_capture_output').with_args(
+        full_command=object,
+        environment=object,
+        working_directory='/working/dir',
+        borg_local_path=object,
+        borg_exit_codes=object,
+    ).once()
+    flexmock(module.flags).should_receive('warn_for_aggressive_archive_flags')
+    flexmock(module).should_receive('execute_command').with_args(
+        full_command=object,
+        output_log_level=object,
+        environment=object,
+        working_directory='/working/dir',
+        borg_local_path=object,
+        borg_exit_codes=object,
+    ).once()
+
+    module.display_archives_info(
+        repository_path='repo',
+        config={'working_directory': '/working/dir'},
+        local_borg_version='2.3.4',
+        global_arguments=flexmock(),
+        info_arguments=flexmock(archive=None, json=False, prefix=None, match_archives=None),
+    )
