@@ -20,7 +20,7 @@ def test_make_list_command_includes_log_info():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--info', 'repo')
@@ -37,7 +37,7 @@ def test_make_list_command_includes_json_but_not_info():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=True),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--json', 'repo')
@@ -54,7 +54,7 @@ def test_make_list_command_includes_log_debug():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--debug', '--show-rc', 'repo')
@@ -71,7 +71,7 @@ def test_make_list_command_includes_json_but_not_debug():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=True),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--json', 'repo')
@@ -87,30 +87,32 @@ def test_make_list_command_includes_json():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=True),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--json', 'repo')
 
 
 def test_make_list_command_includes_log_json():
-    flexmock(module.flags).should_receive('make_flags').and_return(()).and_return(('--log-json',))
+    flexmock(module.flags).should_receive('make_flags').and_return(()).and_return(()).and_return(
+        ('--log-json',)
+    )
     flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(())
     flexmock(module.flags).should_receive('make_repository_flags').and_return(('repo',))
 
     command = module.make_list_command(
         repository_path='repo',
-        config={},
+        config={'log_json': True},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=True),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--log-json', 'repo')
 
 
 def test_make_list_command_includes_lock_wait():
-    flexmock(module.flags).should_receive('make_flags').and_return(()).and_return(
+    flexmock(module.flags).should_receive('make_flags').and_return(()).and_return(()).and_return(
         ('--lock-wait', '5')
     )
     flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(())
@@ -121,7 +123,7 @@ def test_make_list_command_includes_lock_wait():
         config={'lock_wait': 5},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--lock-wait', '5', 'repo')
@@ -139,7 +141,7 @@ def test_make_list_command_includes_archive():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive='archive', paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', 'repo::archive')
@@ -157,7 +159,7 @@ def test_make_list_command_includes_archive_and_path():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive='archive', paths=['var/lib'], json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', 'repo::archive', 'var/lib')
@@ -173,7 +175,7 @@ def test_make_list_command_includes_local_path():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         local_path='borg2',
     )
 
@@ -196,13 +198,31 @@ def test_make_list_command_includes_remote_path():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         remote_path='borg2',
     )
 
     assert command == ('borg', 'list', '--remote-path', 'borg2', 'repo')
 
 
+def test_make_list_command_includes_umask():
+    flexmock(module.flags).should_receive('make_flags').replace_with(
+        lambda name, value: (f'--{name}', value) if value else ()
+    )
+    flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(())
+    flexmock(module.flags).should_receive('make_repository_flags').and_return(('repo',))
+
+    command = module.make_list_command(
+        repository_path='repo',
+        config={'umask': '077'},
+        local_borg_version='1.2.3',
+        list_arguments=flexmock(archive=None, paths=None, json=False),
+        global_arguments=flexmock(),
+    )
+
+    assert command == ('borg', 'list', '--umask', '077', 'repo')
+
+
 def test_make_list_command_includes_short():
     flexmock(module.flags).should_receive('make_flags').and_return(())
     flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(('--short',))
@@ -213,7 +233,7 @@ def test_make_list_command_includes_short():
         config={},
         local_borg_version='1.2.3',
         list_arguments=flexmock(archive=None, paths=None, json=False, short=True),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--short', 'repo')
@@ -252,7 +272,7 @@ def test_make_list_command_includes_additional_flags(argument_name):
             format=None,
             **{argument_name: 'value'},
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
     assert command == ('borg', 'list', '--' + argument_name.replace('_', '-'), 'value', 'repo')
@@ -287,6 +307,7 @@ def test_make_find_paths_adds_globs_to_path_fragments():
 
 def test_capture_archive_listing_does_not_raise():
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command_and_capture_output').and_return('')
     flexmock(module).should_receive('make_list_command')
 
@@ -295,7 +316,7 @@ def test_capture_archive_listing_does_not_raise():
         archive='archive',
         config={},
         local_borg_version=flexmock(),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -314,7 +335,7 @@ def test_list_archive_calls_borg_with_flags():
         first=None,
         last=None,
     )
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
 
     flexmock(module.feature).should_receive('available').and_return(False)
     flexmock(module).should_receive('make_list_command').with_args(
@@ -328,12 +349,14 @@ def test_list_archive_calls_borg_with_flags():
     ).and_return(('borg', 'list', 'repo::archive'))
     flexmock(module).should_receive('make_find_paths').and_return(())
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive'),
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -359,7 +382,7 @@ def test_list_archive_with_archive_and_json_errors():
             config={},
             local_borg_version='1.2.3',
             list_arguments=list_arguments,
-            global_arguments=flexmock(log_json=False),
+            global_arguments=flexmock(),
         )
 
 
@@ -378,7 +401,7 @@ def test_list_archive_calls_borg_with_local_path():
         first=None,
         last=None,
     )
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
 
     flexmock(module.feature).should_receive('available').and_return(False)
     flexmock(module).should_receive('make_list_command').with_args(
@@ -392,12 +415,14 @@ def test_list_archive_calls_borg_with_local_path():
     ).and_return(('borg2', 'list', 'repo::archive'))
     flexmock(module).should_receive('make_find_paths').and_return(())
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg2', 'list', 'repo::archive'),
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg2',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -425,7 +450,7 @@ def test_list_archive_calls_borg_using_exit_codes():
         first=None,
         last=None,
     )
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
 
     flexmock(module.feature).should_receive('available').and_return(False)
     borg_exit_codes = flexmock()
@@ -440,12 +465,14 @@ def test_list_archive_calls_borg_using_exit_codes():
     ).and_return(('borg', 'list', 'repo::archive'))
     flexmock(module).should_receive('make_find_paths').and_return(())
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive'),
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=borg_exit_codes,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -474,10 +501,14 @@ def test_list_archive_calls_borg_multiple_times_with_find_paths():
     )
 
     flexmock(module.feature).should_receive('available').and_return(False)
-    flexmock(module.rlist).should_receive('make_rlist_command').and_return(('borg', 'list', 'repo'))
+    flexmock(module.repo_list).should_receive('make_repo_list_command').and_return(
+        ('borg', 'list', 'repo')
+    )
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command_and_capture_output').with_args(
         ('borg', 'list', 'repo'),
-        extra_environment=None,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
     ).and_return('archive1\narchive2').once()
@@ -489,16 +520,18 @@ def test_list_archive_calls_borg_multiple_times_with_find_paths():
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive1') + glob_paths,
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive2') + glob_paths,
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -506,7 +539,7 @@ def test_list_archive_calls_borg_multiple_times_with_find_paths():
         config={},
         local_borg_version='1.2.3',
         list_arguments=list_arguments,
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -525,7 +558,7 @@ def test_list_archive_calls_borg_with_archive():
         first=None,
         last=None,
     )
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
 
     flexmock(module.feature).should_receive('available').and_return(False)
     flexmock(module).should_receive('make_list_command').with_args(
@@ -539,12 +572,14 @@ def test_list_archive_calls_borg_with_archive():
     ).and_return(('borg', 'list', 'repo::archive'))
     flexmock(module).should_receive('make_find_paths').and_return(())
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive'),
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -574,7 +609,7 @@ def test_list_archive_without_archive_delegates_to_list_repository():
     )
 
     flexmock(module.feature).should_receive('available').and_return(False)
-    flexmock(module.rlist).should_receive('list_repository')
+    flexmock(module.repo_list).should_receive('list_repository')
     flexmock(module.environment).should_receive('make_environment').never()
     flexmock(module).should_receive('execute_command').never()
 
@@ -583,7 +618,7 @@ def test_list_archive_without_archive_delegates_to_list_repository():
         config={},
         local_borg_version='1.2.3',
         list_arguments=list_arguments,
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -605,7 +640,7 @@ def test_list_archive_with_borg_features_without_archive_delegates_to_list_repos
     )
 
     flexmock(module.feature).should_receive('available').and_return(True)
-    flexmock(module.rlist).should_receive('list_repository')
+    flexmock(module.repo_list).should_receive('list_repository')
     flexmock(module.environment).should_receive('make_environment').never()
     flexmock(module).should_receive('execute_command').never()
 
@@ -614,7 +649,7 @@ def test_list_archive_with_borg_features_without_archive_delegates_to_list_repos
         config={},
         local_borg_version='1.2.3',
         list_arguments=list_arguments,
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -634,7 +669,7 @@ def test_list_archive_with_archive_ignores_archive_filter_flag(
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
     flexmock(module.logging).ANSWER = module.borgmatic.logger.ANSWER
     flexmock(module.logger).answer = lambda message: None
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
     default_filter_flags = {
         'prefix': None,
         'match_archives': None,
@@ -645,7 +680,7 @@ def test_list_archive_with_archive_ignores_archive_filter_flag(
     altered_filter_flags = {**default_filter_flags, **{archive_filter_flag: 'foo'}}
 
     flexmock(module.feature).should_receive('available').with_args(
-        module.feature.Feature.RLIST, '1.2.3'
+        module.feature.Feature.REPO_LIST, '1.2.3'
     ).and_return(False)
     flexmock(module).should_receive('make_list_command').with_args(
         repository_path='repo',
@@ -660,12 +695,14 @@ def test_list_archive_with_archive_ignores_archive_filter_flag(
     ).and_return(('borg', 'list', 'repo::archive'))
     flexmock(module).should_receive('make_find_paths').and_return(())
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', 'repo::archive'),
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -689,7 +726,7 @@ def test_list_archive_with_archive_ignores_archive_filter_flag(
         'last',
     ),
 )
-def test_list_archive_with_find_paths_allows_archive_filter_flag_but_only_passes_it_to_rlist(
+def test_list_archive_with_find_paths_allows_archive_filter_flag_but_only_passes_it_to_repo_list(
     archive_filter_flag,
 ):
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
@@ -704,24 +741,26 @@ def test_list_archive_with_find_paths_allows_archive_filter_flag_but_only_passes
     }
     altered_filter_flags = {**default_filter_flags, **{archive_filter_flag: 'foo'}}
     glob_paths = ('**/*foo.txt*/**',)
-    global_arguments = flexmock(log_json=False)
+    global_arguments = flexmock()
     flexmock(module.feature).should_receive('available').and_return(True)
 
-    flexmock(module.rlist).should_receive('make_rlist_command').with_args(
+    flexmock(module.repo_list).should_receive('make_repo_list_command').with_args(
         repository_path='repo',
         config={},
         local_borg_version='1.2.3',
-        rlist_arguments=argparse.Namespace(
+        repo_list_arguments=argparse.Namespace(
             repository='repo', short=True, format=None, json=None, **altered_filter_flags
         ),
         global_arguments=global_arguments,
         local_path='borg',
         remote_path=None,
-    ).and_return(('borg', 'rlist', '--repo', 'repo'))
+    ).and_return(('borg', 'repo-list', '--repo', 'repo'))
 
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command_and_capture_output').with_args(
-        ('borg', 'rlist', '--repo', 'repo'),
-        extra_environment=None,
+        ('borg', 'repo-list', '--repo', 'repo'),
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
     ).and_return('archive1\narchive2').once()
@@ -769,16 +808,18 @@ def test_list_archive_with_find_paths_allows_archive_filter_flag_but_only_passes
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', '--repo', 'repo', 'archive1') + glob_paths,
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'list', '--repo', 'repo', 'archive2') + glob_paths,
         output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     module.list_archive(
@@ -797,3 +838,53 @@ def test_list_archive_with_find_paths_allows_archive_filter_flag_but_only_passes
         ),
         global_arguments=global_arguments,
     )
+
+
+def test_list_archive_calls_borg_with_working_directory():
+    flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
+    flexmock(module.logging).ANSWER = module.borgmatic.logger.ANSWER
+    flexmock(module.logger).answer = lambda message: None
+    list_arguments = argparse.Namespace(
+        archive='archive',
+        paths=None,
+        json=False,
+        find_paths=None,
+        prefix=None,
+        match_archives=None,
+        sort_by=None,
+        first=None,
+        last=None,
+    )
+    global_arguments = flexmock()
+
+    flexmock(module.feature).should_receive('available').and_return(False)
+    flexmock(module).should_receive('make_list_command').with_args(
+        repository_path='repo',
+        config={'working_directory': '/working/dir'},
+        local_borg_version='1.2.3',
+        list_arguments=list_arguments,
+        global_arguments=global_arguments,
+        local_path='borg',
+        remote_path=None,
+    ).and_return(('borg', 'list', 'repo::archive'))
+    flexmock(module).should_receive('make_find_paths').and_return(())
+    flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(
+        '/working/dir',
+    )
+    flexmock(module).should_receive('execute_command').with_args(
+        ('borg', 'list', 'repo::archive'),
+        output_log_level=module.borgmatic.logger.ANSWER,
+        environment=None,
+        working_directory='/working/dir',
+        borg_local_path='borg',
+        borg_exit_codes=None,
+    ).once()
+
+    module.list_archive(
+        repository_path='repo',
+        config={'working_directory': '/working/dir'},
+        local_borg_version='1.2.3',
+        list_arguments=list_arguments,
+        global_arguments=global_arguments,
+    )
