@@ -7,13 +7,17 @@ from borgmatic.borg import mount as module
 from ..test_verbosity import insert_logging_mock
 
 
-def insert_execute_command_mock(command, borg_exit_codes=None):
+def insert_execute_command_mock(command, working_directory=None, borg_exit_codes=None):
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(
+        working_directory,
+    )
     flexmock(module).should_receive('execute_command').with_args(
         command,
+        environment=None,
+        working_directory=working_directory,
         borg_local_path=command[0],
         borg_exit_codes=borg_exit_codes,
-        extra_environment=None,
     ).once()
 
 
@@ -29,7 +33,7 @@ def test_mount_archive_calls_borg_with_required_flags():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -52,7 +56,7 @@ def test_mount_archive_with_borg_features_calls_borg_with_repository_and_match_a
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -70,7 +74,7 @@ def test_mount_archive_without_archive_calls_borg_with_repository_flags_only():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -90,7 +94,7 @@ def test_mount_archive_calls_borg_with_path_flags():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -108,7 +112,7 @@ def test_mount_archive_calls_borg_with_local_path():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         local_path='borg1',
     )
 
@@ -131,7 +135,7 @@ def test_mount_archive_calls_borg_using_exit_codes():
         mount_arguments=mount_arguments,
         config={'borg_exit_codes': borg_exit_codes},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -151,7 +155,7 @@ def test_mount_archive_calls_borg_with_remote_path_flags():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         remote_path='borg1',
     )
 
@@ -170,7 +174,7 @@ def test_mount_archive_calls_borg_with_umask_flags():
         mount_arguments=mount_arguments,
         config={'umask': '0770'},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -186,9 +190,9 @@ def test_mount_archive_calls_borg_with_log_json_flags():
         repository_path='repo',
         archive='archive',
         mount_arguments=mount_arguments,
-        config={},
+        config={'log_json': True},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=True),
+        global_arguments=flexmock(),
     )
 
 
@@ -206,7 +210,7 @@ def test_mount_archive_calls_borg_with_lock_wait_flags():
         mount_arguments=mount_arguments,
         config={'lock_wait': '5'},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -225,7 +229,7 @@ def test_mount_archive_with_log_info_calls_borg_with_info_parameter():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -244,7 +248,7 @@ def test_mount_archive_with_log_debug_calls_borg_with_debug_flags():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -254,12 +258,14 @@ def test_mount_archive_calls_borg_with_foreground_parameter():
         ('repo::archive',)
     )
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         ('borg', 'mount', '--foreground', 'repo::archive', '/mnt'),
         output_file=module.DO_NOT_CAPTURE,
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     ).once()
 
     mount_arguments = flexmock(mount_point='/mnt', options=None, paths=None, foreground=True)
@@ -269,7 +275,7 @@ def test_mount_archive_calls_borg_with_foreground_parameter():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -289,7 +295,7 @@ def test_mount_archive_calls_borg_with_options_flags():
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -312,6 +318,7 @@ def test_mount_archive_with_date_based_matching_calls_borg_with_date_based_flags
     )
     flexmock(module.flags).should_receive('make_repository_flags').and_return(('--repo', 'repo'))
     flexmock(module.environment).should_receive('make_environment')
+    flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
     flexmock(module).should_receive('execute_command').with_args(
         (
             'borg',
@@ -330,9 +337,10 @@ def test_mount_archive_with_date_based_matching_calls_borg_with_date_based_flags
             'repo',
             '/mnt',
         ),
+        environment=None,
+        working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
-        extra_environment=None,
     )
 
     mount_arguments = flexmock(
@@ -351,5 +359,21 @@ def test_mount_archive_with_date_based_matching_calls_borg_with_date_based_flags
         mount_arguments=mount_arguments,
         config={},
         local_borg_version='1.2.3',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
+    )
+
+
+def test_mount_archive_calls_borg_with_working_directory():
+    flexmock(module.feature).should_receive('available').and_return(False)
+    flexmock(module.flags).should_receive('make_repository_flags').and_return(('repo',))
+    insert_execute_command_mock(('borg', 'mount', 'repo', '/mnt'), working_directory='/working/dir')
+
+    mount_arguments = flexmock(mount_point='/mnt', options=None, paths=None, foreground=False)
+    module.mount_archive(
+        repository_path='repo',
+        archive=None,
+        mount_arguments=mount_arguments,
+        config={'working_directory': '/working/dir'},
+        local_borg_version='1.2.3',
+        global_arguments=flexmock(),
     )
