@@ -21,7 +21,7 @@ def test_transfer_archives_calls_borg_with_flags():
         ('borg', 'transfer', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -35,7 +35,7 @@ def test_transfer_archives_calls_borg_with_flags():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -55,7 +55,7 @@ def test_transfer_archives_with_dry_run_calls_borg_with_dry_run_flag():
         ('borg', 'transfer', '--repo', 'repo', '--dry-run'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -69,7 +69,7 @@ def test_transfer_archives_with_dry_run_calls_borg_with_dry_run_flag():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -86,7 +86,7 @@ def test_transfer_archives_with_log_info_calls_borg_with_info_flag():
         ('borg', 'transfer', '--info', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -100,7 +100,7 @@ def test_transfer_archives_with_log_info_calls_borg_with_info_flag():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -117,7 +117,7 @@ def test_transfer_archives_with_log_debug_calls_borg_with_debug_flag():
         ('borg', 'transfer', '--debug', '--show-rc', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -132,7 +132,7 @@ def test_transfer_archives_with_log_debug_calls_borg_with_debug_flag():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -151,7 +151,7 @@ def test_transfer_archives_with_archive_calls_borg_with_match_archives_flag():
         ('borg', 'transfer', '--match-archives', 'archive', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -165,7 +165,7 @@ def test_transfer_archives_with_archive_calls_borg_with_match_archives_flag():
         transfer_arguments=flexmock(
             archive='archive', progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -184,7 +184,7 @@ def test_transfer_archives_with_match_archives_calls_borg_with_match_archives_fl
         ('borg', 'transfer', '--match-archives', 'sh:foo*', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -193,12 +193,12 @@ def test_transfer_archives_with_match_archives_calls_borg_with_match_archives_fl
     module.transfer_archives(
         dry_run=False,
         repository_path='repo',
-        config={'archive_name_format': 'bar-{now}'},  # noqa: FS003
+        config={'archive_name_format': 'bar-{now}', 'match_archives': 'sh:foo*'},  # noqa: FS003
         local_borg_version='2.3.4',
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives='sh:foo*', source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -217,7 +217,7 @@ def test_transfer_archives_with_archive_name_format_calls_borg_with_match_archiv
         ('borg', 'transfer', '--match-archives', 'sh:bar-*', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -231,7 +231,7 @@ def test_transfer_archives_with_archive_name_format_calls_borg_with_match_archiv
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -248,7 +248,7 @@ def test_transfer_archives_with_local_path_calls_borg_via_local_path():
         ('borg2', 'transfer', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg2',
         borg_exit_codes=None,
@@ -262,7 +262,7 @@ def test_transfer_archives_with_local_path_calls_borg_via_local_path():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         local_path='borg2',
     )
 
@@ -281,7 +281,7 @@ def test_transfer_archives_with_exit_codes_calls_borg_using_them():
         ('borg', 'transfer', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=borg_exit_codes,
@@ -295,7 +295,7 @@ def test_transfer_archives_with_exit_codes_calls_borg_using_them():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -315,7 +315,7 @@ def test_transfer_archives_with_remote_path_calls_borg_with_remote_path_flags():
         ('borg', 'transfer', '--remote-path', 'borg2', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -329,7 +329,7 @@ def test_transfer_archives_with_remote_path_calls_borg_with_remote_path_flags():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         remote_path='borg2',
     )
 
@@ -349,7 +349,7 @@ def test_transfer_archives_with_umask_calls_borg_with_umask_flags():
         ('borg', 'transfer', '--umask', '077', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -363,7 +363,7 @@ def test_transfer_archives_with_umask_calls_borg_with_umask_flags():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -383,7 +383,7 @@ def test_transfer_archives_with_log_json_calls_borg_with_log_json_flags():
         ('borg', 'transfer', '--log-json', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -392,12 +392,12 @@ def test_transfer_archives_with_log_json_calls_borg_with_log_json_flags():
     module.transfer_archives(
         dry_run=False,
         repository_path='repo',
-        config={},
+        config={'log_json': True},
         local_borg_version='2.3.4',
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=True),
+        global_arguments=flexmock(),
     )
 
 
@@ -418,7 +418,7 @@ def test_transfer_archives_with_lock_wait_calls_borg_with_lock_wait_flags():
         ('borg', 'transfer', '--lock-wait', '5', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -432,16 +432,19 @@ def test_transfer_archives_with_lock_wait_calls_borg_with_lock_wait_flags():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
-def test_transfer_archives_with_progress_calls_borg_with_progress_flag():
+def test_transfer_archives_with_progress_calls_borg_with_progress_flags():
     flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
     flexmock(module.logging).ANSWER = module.borgmatic.logger.ANSWER
     flexmock(module.flags).should_receive('make_flags').and_return(())
+    flexmock(module.flags).should_receive('make_flags').with_args('progress', True).and_return(
+        ('--progress',)
+    )
     flexmock(module.flags).should_receive('make_match_archives_flags').and_return(())
-    flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(('--progress',))
+    flexmock(module.flags).should_receive('make_flags_from_arguments').and_return(())
     flexmock(module.flags).should_receive('make_repository_flags').and_return(('--repo', 'repo'))
     flexmock(module.environment).should_receive('make_environment')
     flexmock(module.borgmatic.config.paths).should_receive('get_working_directory').and_return(None)
@@ -449,7 +452,7 @@ def test_transfer_archives_with_progress_calls_borg_with_progress_flag():
         ('borg', 'transfer', '--progress', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=module.DO_NOT_CAPTURE,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -458,12 +461,12 @@ def test_transfer_archives_with_progress_calls_borg_with_progress_flag():
     module.transfer_archives(
         dry_run=False,
         repository_path='repo',
-        config={},
+        config={'progress': True},
         local_borg_version='2.3.4',
         transfer_arguments=flexmock(
-            archive=None, progress=True, match_archives=None, source_repository=None
+            archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -484,7 +487,7 @@ def test_transfer_archives_passes_through_arguments_to_borg(argument_name):
         ('borg', 'transfer', flag_name, 'value', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -502,7 +505,7 @@ def test_transfer_archives_passes_through_arguments_to_borg(argument_name):
             source_repository=None,
             **{argument_name: 'value'},
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -521,7 +524,7 @@ def test_transfer_archives_with_source_repository_calls_borg_with_other_repo_fla
         ('borg', 'transfer', '--repo', 'repo', '--other-repo', 'other'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -535,7 +538,7 @@ def test_transfer_archives_with_source_repository_calls_borg_with_other_repo_fla
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository='other'
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
 
 
@@ -566,7 +569,7 @@ def test_transfer_archives_with_date_based_matching_calls_borg_with_date_based_f
         ),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory=None,
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -577,7 +580,7 @@ def test_transfer_archives_with_date_based_matching_calls_borg_with_date_based_f
         repository_path='repo',
         config={},
         local_borg_version='2.3.4',
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
         transfer_arguments=flexmock(
             archive=None,
             progress=None,
@@ -605,7 +608,7 @@ def test_transfer_archives_calls_borg_with_working_directory():
         ('borg', 'transfer', '--repo', 'repo'),
         output_log_level=module.borgmatic.logger.ANSWER,
         output_file=None,
-        extra_environment=None,
+        environment=None,
         working_directory='/working/dir',
         borg_local_path='borg',
         borg_exit_codes=None,
@@ -619,5 +622,5 @@ def test_transfer_archives_calls_borg_with_working_directory():
         transfer_arguments=flexmock(
             archive=None, progress=None, match_archives=None, source_repository=None
         ),
-        global_arguments=flexmock(log_json=False),
+        global_arguments=flexmock(),
     )
